/**
   Copyright (C) 2012 . see<http://mysources.sourceforge.net />

   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses />.
 */

package net.sourceforge.mysources.thinkinjava4.enumerated;

import net.sourceforge.mysources.thinkinjava4.utils.Print;

/**
 * 邮局
 * 
 * 在职责链(Chain of Responsibilty)设计模式中，我们
 * 以多种不同的方式解决一个问题，然后将他们链接在一起，当
 * 一个请求到来时，它遍历这个链，直到链中的某个解决方案能够
 * 处理该请求。
 * 
 * 通过常量相关的方法， 我们可以很容易地实现一个简单的职责链。
 * 我们以一个邮局的模型为例。邮局需要以尽可能通用的方式来处理
 * 每一封邮件，并且要不断尝试处理邮件，直到该邮件最终被确定为
 * 一封死信。其中的每一次尝试可以看作为一个策略（设计模式），
 * 而完整的处理方式列表就是一个职责链。
 *
 * @author Lucky Yang
 * @since 0.1
 */
public class PostOffice {
	enum MailHandler{	
		GENERAL_DELIVERY{
			boolean handle(Mail m){
				switch(m.generalDelivery){
					case YES:
						Print.print("Using general delivery for " + m);
						return true;
					default: return false;
				}
			}
		},
		MACHINE_SCAN{
			boolean handle(Mail m){
				switch(m.scannability){
					case UNSCANNABLE:
						return false;
					default: 
						Print.print("Delivering " + m + " automatically");
						return true;
				}
			}			
		},
		VISUAL_INSPECTION{
			boolean handle(Mail m){
				switch(m.readability){
					case ILLEGIBLE:
						return false;
					default:
						switch(m.address){
							case INCORRECT:
								return false;
							default: 
								Print.print("Delivering " + m + " normally");
								return true;
						}
				}
			}				
		},
		RETURN_TO_SENDER{
			boolean handle(Mail m){
				switch(m.returnAddress){
					case MISSING:
						return false;
					default:
						Print.print("Delivering " + m + " to sender");
						return true;
				}
			}							
		};
		
		abstract boolean handle(Mail m);
	}
	
	public static void handle(Mail m){
		for(MailHandler handler : MailHandler.values())
			if (handler.handle(m)) 
				return;
		
		Print.print(m + " is a dead letter");
	}
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		for(Mail mail : Mail.generator(500)){
			Print.print(mail.details());
			PostOffice.handle(mail);
			Print.print("*****");
		}

	}	
}
